@@ -8,6 +8,7 @@ from aiida.orm import load_node, QueryBuilder, Node
 from copy import deepcopy
 from hzdplugins.aiidaplugins.constants import results_keys_set
 import json
+import qeschema
 
 def getChargeAndMagneticMoments(uuid, traj_index=-1):
     """
@@ -457,7 +458,7 @@ def checkDistance(cell, atom1, atom2, bond_length):
                     # return True, distance(atom1.position, atom2_modifyPosition), 'ax:{}, ay:{}, az:{}'.format(ax, ay,
                     #                                                                                           az)
     if len(results) != 0:
-        return results 
+        return results
     else:
         return [(False, -1, '')]
 
@@ -487,7 +488,7 @@ def getGCN(structure, bond_length, atom_list, cn_max):
 
     :param structure: Structure of the slab
     :type structure: aiida.orm.StructureData
-    
+
     :param bond_length: the maximum bond_length that we need to investigate
     :type bond_length: python real object
 
@@ -505,7 +506,7 @@ def getGCN(structure, bond_length, atom_list, cn_max):
 
     # get the coordination dictionary
     coord_dict = getStructureAnalysis(structure=structure, atom_index=atom_list, bond_length = bond_length, is_Metal=False)
-    
+
     # do the analysis
 
     conn_set = [] # for all the atoms that connected to the site
@@ -516,14 +517,59 @@ def getGCN(structure, bond_length, atom_list, cn_max):
                 items = match.groups()
             index = int(items[1]) # get the index of that atom
             conn_set.append(index)
-    
-    print(conn_set)
+
     sum_cn = 0
     for ind in conn_set:
         coord_dict = getStructureAnalysis(structure=structure, atom_index=[ind], bond_length=bond_length, is_Metal=False)
         for key, value in coord_dict.items():
             sum_cn += len(value)
-    
+
     return sum_cn/cn_max
 
+def checkTotalSpin(symbol, electron_configuration):
+    """
+    Get the spin of certain configurations.
+    """
+    results = 0
+    for line in electron_configuration:
+        if line[1] == 1:
+            results += line[3]*1.0
+        else:
+            results += line[3]*(-1.0)
+    return results
 
+def getXMLFromPW(xml_file):
+    """
+    Get the dictionary from the xml output.
+
+    :param xml_file: The location of the .xml file that we need
+    :returns: return the python dictionary that contains all the information
+    """
+
+    pw_document = qeschema.PwDocument()
+    pw_document.read(xml_file)
+    dict_data = pw_document.to_dict()
+    
+    return dict_data
+
+def getOptimizedStructure(data_xml):
+    """
+    Get the last structure (atomic positions and cell) from the xml data (parsed as dictionary)
+
+    :param data_xml: The python dictionary parsed from the xml file generated by pw.x
+    :returns: There are two returns, the first is the cell of the optimized structure, the second is the atomic positions.
+    """
+
+    cell = data_xml['qes:espresso']['output']['atomic_structure']['cell']
+    tmp_cell = []
+    for key, value in cell.itemize():
+        tmp_cell.append(value)
+    
+    atomic_positions = data_xml['qes:espresso']['output']['atomic_structure']['atomic_positions']['atom']
+    tmp_atomic_positions = []
+    for atom in atomic_positions:
+        tmp_list = [atom['@name'], atom['$'][0], atom['$'][1], atom['$'][2]]
+        tmp_atomic_positions.append(tmp_list)
+    
+    return tmp_cell, tmp_atomic_positions
+    
